clear all; close all; clc; format long; graphTol=0.05;

% Max number of persistent components 
JMAX=10;

load('USD_CHF_RV_89_93')
T=length(rvfx);
muR=mean(rvfx);
chronR=rvfx-muR;    %chron denotes chronological order
r = flipud(chronR); % demeaned rvfx

% choice of maxAR via BIC
BIC=zeros(50,1);        

for p=1:length(BIC)      %parameters of the AR 
    truncR=r(1:end-p);   
    Rmat=zeros(length(truncR),p); 
    for i=1:length(truncR)
        Rmat(i,:)=r(i+1:i+p); %the ith row has p steps ahead of r 
    end
    [~,~,R,~,~]=regress(truncR,Rmat);   
    sigmaML= (R'*R)/length(truncR);      % error variance  
    % assume that errors are Gaussian
    % compute the log likelihood 
    l_b=-(length(truncR)/2)*log(2*pi*sigmaML)-length(truncR)/2;
    %BIC 
    BIC(p)=-2*l_b + p*log(length(truncR));
end
[~,maxAR]=min(BIC); 
clear p truncR Rmat sigmaML l_b BIC  

% AR REGRESSION with maxAR lags
truncR=r(1:end-maxAR);
Rmat = zeros(length(truncR),maxAR); 
for i=1:length(truncR)
    Rmat(i,:)=r(i+1:i+maxAR); 
end
[b,~,Eta]=regress(truncR,Rmat);   % residuals are in Eta
sigma2= (Eta'*Eta)/(length(truncR)-maxAR); % error variance 
sigma=sqrt(sigma2);
Eps=Eta/sigma;                    % unit variance residuals Eps
clear truncR Rmat sigma2

% Classical Wold coefficients alphaR
% calculated as in Sect 5 of Di Virgilio, Ortu, Severino, Tebaldi 2019
% M lenght of alphaR: alphaR_0,...,alphaR_{M-1}  
M=2^(JMAX)*(floor((T-maxAR)/(2^(JMAX)))-1);
alphaR=zeros(M,1); 
alphaR(1)=sigma;
if maxAR~=0  
    for n=1:(length(alphaR)-1) 
        hstart=max(n-maxAR,0); 
        temp=0;
        for h=hstart:n-1 
            temp=temp+alphaR(h+1)*b(n-h); 
        end
        alphaR(n+1)=temp;
    end
end
clear sigma hstart h

%vector betaRJ of multiscale IRF at scale J has length length(alphaR)/(2^J)
%vector rJ of (demeaned) component at scale J in reverse order with length T-maxAR-KMAX+1

KMAX=M;

[betaR1, ~, r1, ~, ~]=IRFscale(T,maxAR,alphaR,Eps,KMAX,1);
[betaR2, ~, r2, ~, ~]=IRFscale(T,maxAR,alphaR,Eps,KMAX,2);
[betaR3, ~, r3, ~, ~]=IRFscale(T,maxAR,alphaR,Eps,KMAX,3);
[betaR4, ~, r4, ~, ~]=IRFscale(T,maxAR,alphaR,Eps,KMAX,4);    
[betaR5, ~, r5, ~, ~]=IRFscale(T,maxAR,alphaR,Eps,KMAX,5);
[betaR6, ~, r6, ~, ~]=IRFscale(T,maxAR,alphaR,Eps,KMAX,6);
[betaR7, ~, r7, ~, ~]=IRFscale(T,maxAR,alphaR,Eps,KMAX,7);
[betaR8, ~, r8, ~, ~]=IRFscale(T,maxAR,alphaR,Eps,KMAX,8);
[betaR9, ~, r9, ~, ~]=IRFscale(T,maxAR,alphaR,Eps,KMAX,9);
[betaR10, ~, r10, ~, ~]=IRFscale(T,maxAR,alphaR,Eps,KMAX,10);

% explained variance by scales
variance=zeros(10,1);
varianceRatio=zeros(10,1);

variance(1)=betaR1'*betaR1;
variance(2)=betaR2'*betaR2;
variance(3)=betaR3'*betaR3;
variance(4)=betaR4'*betaR4;
variance(5)=betaR5'*betaR5;
variance(6)=betaR6'*betaR6;
variance(7)=betaR7'*betaR7;
variance(8)=betaR8'*betaR8;
variance(9)=betaR9'*betaR9;
variance(10)=betaR10'*betaR10;

for i=1:1:10
    varianceRatio(i)=variance(i)/var(r);    % weight over the whole sample variance
end

figure()
plot(1:10,varianceRatio,'-.*')
xlabel('Scale');
ylabel('Scale Variance over total ');
ylim([0 0.3])
title('Relative variance of daily RV','fontname','garamond','fontsize',18);

%%One-step-ahead in-sample forecasts from formulae in Appendix of Ortu Severino Tamoni Tebaldi
[~,rf1]=IRFforecast(T,maxAR,alphaR,Eps,KMAX,1);
[~,rf2]=IRFforecast(T,maxAR,alphaR,Eps,KMAX,2);
[~,rf3]=IRFforecast(T,maxAR,alphaR,Eps,KMAX,3);
[~,rf4]=IRFforecast(T,maxAR,alphaR,Eps,KMAX,4);
[~,rf5]=IRFforecast(T,maxAR,alphaR,Eps,KMAX,5);
[~,rf6]=IRFforecast(T,maxAR,alphaR,Eps,KMAX,6);
[~,rf7]=IRFforecast(T,maxAR,alphaR,Eps,KMAX,7);
[~,rf8]=IRFforecast(T,maxAR,alphaR,Eps,KMAX,8);
[~,rf9]=IRFforecast(T,maxAR,alphaR,Eps,KMAX,9);
[~,rf10]=IRFforecast(T,maxAR,alphaR,Eps,KMAX,10);

%%we run ten regressions, adding every time the scale that explains
%%the most variance

% We will pick up regressors and forecasts from matrices with all the scales
A = [r1 r2 r3 r4 r5 r6 r7 r8 r9 r10];
AF = [rf1 rf2 rf3 rf4 rf5 rf6 rf7 rf8 rf9 rf10];

varRatio = varianceRatio; %temporary variable for explained variance, has scale i at place i

%E matrix for forecast errors (RMSE,MAE,R2MZ)
E=zeros(10,3);

%the matrix of regressors is initially (before iterations)
r_cut = r(1:length(rf1),1); %shortens r to the last T-maxAR-KMAX
Rmat = ones(length(r_cut),1);
Fmat = ones(length(r_cut),1);

iteration =1;
while iteration <10.5 
    %first I need to know which scale to pick
    [~,i]=max(varRatio); %i is the scale to add

    Rmat = [Rmat A(:,i)]; %add the ith scale as regressor
    %Estimate coefficients of regression
    bw=regress(r_cut, Rmat); 

    %Now we make forecasts with this model
    Fmat = [Fmat AF(:,i)]; %adds the ith scale forecast to the previous Fmat
    rforecast_temp = Fmat * bw; %we use the equation with the beta coefficients just estimated, and forecasted regressors

    %RMSE
    ss= 0;
    for q=1:1:(length(r_cut)-1)
        ss= ss + (rforecast_temp(q+1)-r_cut(q))^2;   % forecasts refer to the following day 
    end
    rmse_temp= sqrt(ss/(length(rforecast_temp)-1));
    E(iteration,1) = rmse_temp; %RMSE is in first column of matrix E
    clear ss rmse_temp;

    %MAE
    abserr= 0;
    for q=1:1:(length(r_cut)-1)
        abserr= abserr + abs(rforecast_temp(q+1)-r_cut(q));  % forecasts refer to the following day
    end
    mae_temp = abserr/(length(rforecast_temp)-1);
    E(iteration,2) = mae_temp; %MAE is in second column of matrix E
    clear abserr mae_temp q;

    %Mincer-Zarnowitz regression with our forecasts
    % r(t+1) = a + b*Et((t+1)) + error
    %we regress r_cut on rforecast_temp, paying attention to the fact that
    %rf(h) forecasts the obs of time h-1
    MZreg = [ones(length(rforecast_temp)-1,1) rforecast_temp(2:end)]; 
    [~,~,~,~,statsmz]=regress(r_cut(1:end-1),MZreg); 
    E(iteration,3) = statsmz(1); %R2MZ is in third column of matrix E
    clear statsmz MZreg;

    varRatio(i)=0; %so next max will pick the next greatest value
    iteration=iteration + 1;
end
clear varRatio A AF Rmat Fmat rforecast_temp m i iteration;

figure()
set(gca,'fontname','garamond','fontsize',12); %adjusts font in plots
%plot of E for every scale
yyaxis left 
plot([1:10],E(:,1),'-*','MarkerSize',12); hold on; ylim([1.3 3.3])
plot([1:10],E(:,2),'-o','MarkerSize',12); ylabel('Forecast errors')
yyaxis right  
plot([1:10],E(:,3),'<--','MarkerSize',12); ylim([0 0.65])
ylabel('R2')
xlabel('Number of scales in the model');
legend('RMSE','MAE','R2');
title('Forecast error measures','fontname','garamond','fontsize',18);