% input:  vector alpha of classical Wold innovations
%         with length 2^JMAX * constant
%         T sample length
%         maxAR max lag in the baseline AR
%         Eps vector of unit variance classical Wold innovations in reverse order
%         KMAX=2^(JMAX+3) maximum lag on scales
%         J scale
% output: vector betaPlus of multiscale IRF Beta k,1 at scale J with length length(alpha)/(2^J) 
%         (see Appendix of Ortu Severino Tamoni Tebaldi)
%         vector EpsScale of details at scale J in reverse order with length T-maxAR-2^J+1
%         vector gScale of component forecast at scale J in reverse order with length T-maxAR-KMAX+1

% all processes have ZERO MEAN

function[betaPlus, gScale]=IRFforecast(T,maxAR,alpha,Eps,KMAX,J)
M=length(alpha);
betaPlus=zeros(M/(2^J)-1,1);
for k=0:(floor(M/(2^J))-2)
    betaPlus(k+1)=(sum(alpha(k*2^J+2:k*2^J+2^(J-1)+1)) - sum(alpha(k*2^J+2^(J-1)+2:k*2^J+2^J +1)))/sqrt(2^J);
end

EpsScale=zeros(T-maxAR-2^J+1,1);
for t=0:1:(length(EpsScale)-1)
    EpsScale(t+1)= (sum(Eps(t+1:t+2^(J-1)))-sum(Eps(t+2^(J-1)+1:t+2^J)))/sqrt(2^J);
end

gScale=zeros(T-maxAR-KMAX+1,1);
for t=0:1:(T-maxAR-KMAX)
    for k=0:1:(KMAX/(2^J)-2)
        gScale(t+1)=gScale(t+1)+betaPlus(k+1)*EpsScale(t+k*2^J+1);
    end
end 

return