% input:  vector alpha of classical Wold innovations
%         with length 2^JMAX * constant
%         T sample length
%         maxAR max lag in the baseline AR
%         Eps vector of unit variance classical Wold innovations in reverse order
%         KMAX=2^(JMAX+3) maximum lag on scales
%         J scale
% output: vector betaScale of multiscale IRF at scale J with length length(alpha)/(2^J)
%         vector EpsScale of details at scale J in reverse order with length T-maxAR-2^J+1
%         vector gScale of component at scale J in reverse order with length T-maxAR-KMAX+1
%         vector chronGScale is gScale in chronological order
%         vector decimGScale of decimated component at scale J in reverse order with length floor(length(gScale)/2^J)

% all processes have ZERO MEAN

function[betaScale,EpsScale,gScale,chronGScale,decimGScale]=IRFscale(T,maxAR,alpha,Eps,KMAX,J)
M=length(alpha);
betaScale=zeros(M/(2^J),1);
for k=0:(floor(M/(2^J))-1)
    betaScale(k+1)=(sum(alpha(k*2^J+1:k*2^J+2^(J-1))) - sum(alpha(k*2^J+2^(J-1)+1:k*2^J+2^J)))/sqrt(2^J);
end

EpsScale=zeros(T-maxAR-2^J+1,1);
for t=0:1:(length(EpsScale)-1)
    EpsScale(t+1)= (sum(Eps(t+1:t+2^(J-1)))-sum(Eps(t+2^(J-1)+1:t+2^J)))/sqrt(2^J);
end

gScale=zeros(T-maxAR-KMAX+1,1); 
for t=0:1:(T-maxAR-KMAX)
    for k=0:1:(KMAX/(2^J)-1)
        gScale(t+1)=gScale(t+1)+betaScale(k+1)*EpsScale(t+k*2^J+1);
    end
end 
chronGScale=flipud(gScale);

decimGScale=zeros(floor(length(gScale)/2^J),1);
for i=1:1:length(gScale)
    if mod(i-1,2^J)==0
        decimGScale((i-1)/2^J +1)=gScale(i);
    end
end

return
