% input:  vector alpha of classical Wold innovations
%         with length 2^JMAX * constant
%         T sample length
%         maxAR max lag in the baseline AR
%         Eps vector of unit variance classical Wold innovations in reverse order
%         KMAX=2^(JMAX+3) maximum lag on scales
%         J scale
%         horizon max lag in forecasts
% output: matrix betaPlus of multiscale IRF Beta k,p at scale J with length length(alpha)/(2^J), p goes from 1 to horizon 
%         (see Appendix of Ortu Severino Tamoni Tebaldi)
%         vector gScale of forecast for the sum of following week values of scale J in reverse order with length T-maxAR-KMAX+1

% all processes have ZERO MEAN

function[betaPlus, gScale]=IRFforecast_horizon(T,maxAR,alpha,Eps,KMAX,J,horizon)
    M=length(alpha);
    betaPlus=zeros(floor((M-horizon)/(2^J))-1,horizon); %collects betak,p
    for p=1:horizon %p are the steps ahead as in Appendix of Ortu Severino Tamoni Tebaldi
        for k=0:(floor((M-horizon)/(2^J))-2)
            betaPlus(k+1,p)=(sum(alpha(k*2^J+1+p:k*2^J+2^(J-1)+p)) - sum(alpha(k*2^J+2^(J-1)+1+p:k*2^J+2^J +p)))/sqrt(2^J);
        end
    end
    
    EpsScale=zeros(T-maxAR-2^J+1,1);
    for t=0:1:(length(EpsScale)-1)
        EpsScale(t+1)= (sum(Eps(t+1:t+2^(J-1)))-sum(Eps(t+2^(J-1)+1:t+2^J)))/sqrt(2^J);
    end

    gScale=zeros(T-maxAR-KMAX+1,horizon); %now gscale has to contain all the p step ahead forecasts
    for p=1:horizon
        for t=0:1:(T-maxAR-KMAX)    
            for k=0:1:(floor((KMAX-horizon)/(2^J))-2)
                gScale(t+1,p)=gScale(t+1,p)+betaPlus(k+1,p)*EpsScale(t+k*2^J+1);
            end
        end
    end 
    
    gScale=sum(gScale,2); %we make row-wise sums
return