clear all; close all; clc; format long;
JMAX=9;  %max scale
load('USD_CHF_RV_89_93')

T=length(rvfx);
muR=mean(rvfx);
chronR=rvfx-muR;
r = flipud(chronR);     % demeaned rvfx

%Corsi variables
RVd = r(1:end-22);
%Weekly RV
RVw = zeros(length(RVd),1);
for i=1:length(RVd)
   RVw(i)= (r(i)+r(i+1)+r(i+2)+r(i+3)+r(i+4))/5;
end
%Monthly RV
RVm = zeros(length(RVd),1);
for i=1:length(RVd) 
   temp=0;
   for h=0:21
       temp = temp + r(i+h);
   end
   RVm(i)= temp/22;
   clear temp;
end

iter=0;
errors_corsi=zeros(2,3);
errors_3comp=zeros(2,3);
errors_9comp=zeros(2,3);
TT=2600;

for horizon=[1,66]
    iter=iter+1;
    %means to compare forecasts
    RVh = zeros(length(r)-horizon+1,1);
    for i=1:length(RVh)
        RVh(i)=mean(r(i:i+horizon-1));
    end

    %%OUT of SAMPLE: we re-estimate the models on a moving window
    fcast_length = 3599-TT-21-horizon; %length of forecast sample
    
%% HAR MODEL CORSI 2009 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    horizon_forecast_corsi=zeros(fcast_length,1);
    daily_onestep=zeros(fcast_length,horizon);
    
    SE_corsi=zeros(fcast_length,1); % squared error
    AE_corsi=zeros(fcast_length,1); % absolute error

    for ii=0:fcast_length-1
        RVd_estim = RVd(fcast_length+horizon-ii: fcast_length + TT+horizon-1-ii);  %is the moving window of TT obs, moves towards RVd(0)
        RVw_estim = RVw(fcast_length+horizon-ii: fcast_length + TT+horizon-1-ii);
        RVm_estim = RVm(fcast_length+horizon-ii: fcast_length + TT+horizon-1-ii);

        %regressor matrix with constant
        Rmat = [ones(length(RVd_estim)-1,1) RVd_estim(2:end) RVw_estim(2:end) RVm_estim(2:end)];

        %Model regression
        [beta,bintw,rw,rintw,statsw]=regress(RVd_estim(1:end-1), Rmat); %AR1 structure
        clear bintw rw rintw statsw;
        
        %One-step ahead out of sample forecast, forecasts RVd(fcast_length+4)
        daily_onestep(ii+1,1)= beta(1) + beta(2)*RVd_estim(1) + beta(3)*RVw_estim(1) + beta(4)*RVm_estim(1); %first one is easy as regressors are all in the estim sample
        %from the second on you have to plug the new forecast into all the components, weekly and monthly included 
        if horizon >=2 
            for p=2:min(5,horizon)
                daily_onestep(ii+1,p)= beta(1) + beta(2)*daily_onestep(ii+1,p-1) + beta(3)*0.2*(sum(daily_onestep(ii+1,1:p-1))+sum(RVd_estim(1:5-p+1))) + beta(4)*(1/22)*(sum(daily_onestep(ii+1,1:p-1))+ sum(RVd_estim(1:22-p+1)));
                %you incorporate into the weekly and monthly components the new daily
                %forecasts you have. You have to divide by 5 and 22
            end
        end
        if horizon >= 6
            for p=6:min(22,horizon)
                daily_onestep(ii+1,p)= beta(1) + beta(2)*daily_onestep(ii+1,p-1) + beta(3)*0.2*(sum(daily_onestep(ii+1,p-5:p-1))) + beta(4)*(1/22)*(sum(daily_onestep(ii+1,1:p-1))+ sum(RVd_estim(1:22-p+1)));
            end
        end
        if horizon>= 23
            for p=23:horizon
                daily_onestep(ii+1,p)= beta(1) + beta(2)*daily_onestep(ii+1,p-1) + beta(3)*0.2*(sum(daily_onestep(ii+1,p-5:p-1))) + beta(4)*(1/22)*(sum(daily_onestep(ii+1,p-22:p-1)));
            end
        end
        horizon_forecast_corsi(ii+1) = horizon^(-1)*sum(daily_onestep((ii+1),:)); %make mean row wise to get the mean forecast for horizon

        %Squared error
        SE_corsi(ii+1)=  (horizon_forecast_corsi(ii+1)-RVh(fcast_length-ii))^2 ;
        %Absolute error
        AE_corsi(ii+1)= abs(horizon_forecast_corsi(ii+1)-RVh(fcast_length-ii));
    end

    %Root Mean Squared error
    RMSE_horizon_corsi=sqrt(mean(SE_corsi));
    %Mean Absolute Error
    MAE_horizon_corsi=mean(AE_corsi);
    
    %Mincer-Zarnowitz regression 
    % r(t+1) = a + b*Et((t+1)) + error
    %We basically regress r_cut on rforecast
    MZreg_corsi = [ones(length(horizon_forecast_corsi),1) flipud(horizon_forecast_corsi)]; %need flipud as daily_onestep starts from the last obs
    [~,~,~,~,statsmz]=regress(RVh(1:fcast_length), MZreg_corsi); 
    RsqMZ_horizon_corsi = statsmz(1);

    errors_corsi(iter,:)=[RMSE_horizon_corsi MAE_horizon_corsi RsqMZ_horizon_corsi];
    clear SE_corsi AE_corsi RMSE_horizon_corsi MAE_horizon_corsi MZreg_corsi statsmz RsqMZ_horizon_corsi 
    
%% PERSISTENCE-BASED MODEL WITH 3 AND 9 COMPONENTS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    horizon_forecast_3comp= zeros(fcast_length,1);
    horizon_forecast_9comp= zeros(fcast_length,1);

    SE_3=zeros(fcast_length,1);
    SE_9=zeros(fcast_length,1);
    AE_3=zeros(fcast_length,1);
    AE_9=zeros(fcast_length,1);

    for ii=0:fcast_length-1 %this makes the window move
        est_sample = r(fcast_length+horizon - ii: fcast_length+TT+horizon-1-ii);%you need horizon obs at the end to forecast, last 500 obs are for forecasting

        maxAR=25; % order of AR

% AR regression to estimate the AR coefficients
        truncR=est_sample(1:end-maxAR);
        Rmat = zeros(length(truncR),maxAR); 
        for i=1:length(truncR)
            Rmat(i,:)=est_sample(i+1:i+maxAR); %fill the matrix with lags
        end
        [b,BINT,Eta,RINT,STATS]=regress(truncR,Rmat);   % residuals are in Eta
        sigma2= (Eta'*Eta)/(length(truncR)-maxAR);      % error variance estimation
        sigma=sqrt(sigma2);
        Eps=Eta/sigma;                                  % unit variance residuals Eps
        clear truncR Rmat 

        % Classical Wold coefficients
        % M lenght of alpha: alpha_0,...,alpha_{M-1}   
        M=2^(JMAX)*(floor((TT -maxAR)/(2^(JMAX)))-1);
        KMAX = M;
        %alphas are calculated with the formulas in Sect 5 of Di Virgilio, Ortu, Severino, Tebaldi 2019 
        alphaR=zeros(M,1);
        alphaR(1)=sigma;
       
        for n=1:(length(alphaR)-1) 
            hstart=max(n-maxAR,0);
            temp=0;
            for h=hstart:n-1 
                temp=temp+alphaR(h+1)*b(n-h); 
            end
            alphaR(n+1)=temp;
        end
        clear sigma

        %Now we have the alpha coefficients and the Eps for the current sample. 
        %We estimate the model on the estimation sample
        
        [betaR1, ~, r1, ~, ~]=IRFscale(TT,maxAR,alphaR,Eps,KMAX,1);
        [betaR2, ~, r2, ~, ~]=IRFscale(TT,maxAR,alphaR,Eps,KMAX,2);
        [betaR3, ~, r3, ~, ~]=IRFscale(TT,maxAR,alphaR,Eps,KMAX,3);
        [betaR4, ~, r4, ~, ~]=IRFscale(TT,maxAR,alphaR,Eps,KMAX,4);
        [betaR5, ~, r5, ~, ~]=IRFscale(TT,maxAR,alphaR,Eps,KMAX,5);
        [betaR6, ~, r6, ~, ~]=IRFscale(TT,maxAR,alphaR,Eps,KMAX,6);
        [betaR7, ~, r7, ~, ~]=IRFscale(TT,maxAR,alphaR,Eps,KMAX,7);
        [betaR8, ~, r8, ~, ~]=IRFscale(TT,maxAR,alphaR,Eps,KMAX,8);
        [betaR9, ~, r9, ~, ~]=IRFscale(TT,maxAR,alphaR,Eps,KMAX,9);
        
        Rmat_3comp = [ones(length(r7),1) r7 r8 r9];
        %Estimate coefficients of regression model with 3 components
        [b_3comp,~,~,~,~]=regress(est_sample(1:length(r7)), Rmat_3comp); 
        
        Rmat_9comp = [ones(length(r7),1) r1 r2 r3 r4 r5 r6 r7 r8 r9];
        %Estimate coefficients of regression model with 9 components
        [b_9comp,~,~,~,~]=regress(est_sample(1:length(r7)), Rmat_9comp); 

        %We forecast the sum horizon-step ahead 
        [betaF1,rf1]=IRFforecast_horizon(TT,maxAR,alphaR,Eps,KMAX,1,horizon); 
        [betaF2,rf2]=IRFforecast_horizon(TT,maxAR,alphaR,Eps,KMAX,2,horizon); 
        [betaF3,rf3]=IRFforecast_horizon(TT,maxAR,alphaR,Eps,KMAX,3,horizon); 
        [betaF4,rf4]=IRFforecast_horizon(TT,maxAR,alphaR,Eps,KMAX,4,horizon); 
        [betaF5,rf5]=IRFforecast_horizon(TT,maxAR,alphaR,Eps,KMAX,5,horizon); 
        [betaF6,rf6]=IRFforecast_horizon(TT,maxAR,alphaR,Eps,KMAX,6,horizon); 
        [betaF7,rf7]=IRFforecast_horizon(TT,maxAR,alphaR,Eps,KMAX,7,horizon); 
        [betaF8,rf8]=IRFforecast_horizon(TT,maxAR,alphaR,Eps,KMAX,8,horizon);
        [betaF9,rf9]=IRFforecast_horizon(TT,maxAR,alphaR,Eps,KMAX,9,horizon);

        %first step forecasts for the given horizon average with 3 comp
        RFmat_3comp = [ones(length(rf7),1) rf7 rf8 rf9]; 
        horizon_forecast_3comp(ii+1)= horizon^(-1)* RFmat_3comp(1,:)*[horizon*b_3comp(1); b_3comp(2:end)];
        
        %first step forecasts for the given horizon average with 9 comp
        RFmat_9comp = [ones(length(rf7),1) rf1 rf2 rf3 rf4 rf5 rf6 rf7 rf8 rf9]; 
        horizon_forecast_9comp(ii+1)= horizon^(-1)* RFmat_9comp(1,:)*[horizon*b_9comp(1); b_9comp(2:end)];

        %Squared error with 3 components
        SE_3comp(ii+1)=  (horizon_forecast_3comp(ii+1)-RVh(fcast_length-ii))^2 ; 
        %AE with 3 components
        AE_3comp(ii+1)= abs(horizon_forecast_3comp(ii+1)-RVh(fcast_length-ii));
        %Squared error with 9 components
        SE_9comp(ii+1)=  (horizon_forecast_9comp(ii+1)-RVh(fcast_length-ii))^2 ; 
        %AE with 9 components
        AE_9comp(ii+1)= abs(horizon_forecast_9comp(ii+1)-RVh(fcast_length-ii));
    end

    %RMSE with 3 components
    RMSE_horizon_3comp = sqrt(mean(SE_3comp));
    %MAE with 3 components
    MAE_horizon_3comp = mean(AE_3comp);
    
    %Mincer-Zarnowitz regression with 3 components
    % r(t+1) = a + b*Et((t+1)) + error
    %We basically regress r on rforecast
    MZreg_3comp = [ones(length(horizon_forecast_3comp),1) flipud(horizon_forecast_3comp)]; %you need flipud as daily starts from oldest forecast
    [~,~,~,~,statsmz_3comp]=regress(RVh(1:fcast_length), MZreg_3comp); 
    RsqMZ_horizon_3comp = statsmz_3comp(1);
    
    %RMSE with 9 components
    RMSE_horizon_9comp = sqrt(mean(SE_9comp));
    %MAE with 9 components
    MAE_horizon_9comp = mean(AE_9comp);
    
    %Mincer-Zarnowitz regression with 9 components
    % r(t+1) = a + b*Et((t+1)) + error
    %We basically regress r on rforecast
    MZreg_9comp = [ones(length(horizon_forecast_9comp),1) flipud(horizon_forecast_9comp)]; %you need flipud as daily starts from oldest forecast
    [~,~,~,~,statsmz_9comp]=regress(RVh(1:fcast_length), MZreg_9comp); 
    RsqMZ_horizon_9comp = statsmz_9comp(1);

    errors_3comp(iter,:)=[RMSE_horizon_3comp MAE_horizon_3comp RsqMZ_horizon_3comp];
    errors_9comp(iter,:)=[RMSE_horizon_9comp MAE_horizon_9comp RsqMZ_horizon_9comp];

    clear SE_3comp SE_9comp AE_3comp AE_9comp RMSE_horizon_3comp RMSE_horizon_9comp ...
     statsmz_3comp statsmz_9comp RsqMZ_horizon_3comp RsqMZ_horizon_9comp ... 
     MZreg_3comp MZreg_9comp MAE_horizon_3comp MAE_horizon_9comp;
 
    if horizon==1
        % correlation between forecasts with HAR and forecasts with 3 components
        corr_HAR_EW3comp_1day_matrix=corrcoef(daily_onestep(:,1),horizon_forecast_3comp);
        corr_HAR_EW3comp_1day=corr_HAR_EW3comp_1day_matrix(1,2);
        % correlation between forecasts with HAR and forecasts with 9 components
        corr_HAR_EW9comp_1day_matrix=corrcoef(daily_onestep(:,1),horizon_forecast_9comp);
        corr_HAR_EW9comp_1day=corr_HAR_EW9comp_1day_matrix(1,2);
        clear corr_HAR_EW3comp_1day_matrix corr_HAR_EW9comp_1day_matrix
    end
end

tableOSTT_RMSE = [errors_corsi(:,1), errors_3comp(:,1) errors_9comp(:,1)]';
T_errors_RMSE = array2table(tableOSTT_RMSE, 'VariableNames',{'h1','h66'},'RowNames',{'RMSE_HAR', 'RMSE_EW_3', 'RMSE_EW_9'})

tableOSTT_MAE = [errors_corsi(:,2), errors_3comp(:,2) errors_9comp(:,2)]';
T_errors_MAE = array2table(tableOSTT_MAE, 'VariableNames',{'h1','h66'},'RowNames',{'MAE_HAR', 'MAE_EW_3', 'MAE_EW_9'})

tableOSTT_R2 = [errors_corsi(:,3), errors_3comp(:,3) errors_9comp(:,3)]';
T_errors_R2 = array2table(tableOSTT_R2, 'VariableNames',{'h1','h66'},'RowNames',{'R2_HAR', 'R2_EW_3', 'R2_EW_9'})

corr_HAR_EW3comp_1day
corr_HAR_EW9comp_1day
 