function [zzComponents, const, WoldCoeffEstimated, stdWoldInnovation] = ComputeWoldComponentsFromVAR(z,nlags,Hmax, J)
% PURPOSE: Compute components from Wold decompositon
% 
% -----------------------------------------------------
% USAGE: [yyComponents, const] = ComputeWoldComponents(z,p,Hmax, J)
% where:
% z    = 
% p    = Compute shocks from VAR(p)
% Hmax =
% J    = number of components (actually J+1 since we have also \pi).
%  
% -----------------------------------------------------
% RETURNS: 
% yyComponents = the j-th column is associated with the j-th component, ordered as the shocks.
% const        =
% 
% WoldCoeffEstimated = 
% sum(yyComponents,2) ~ yy (original series)



%-------------------- Compute Wold Innovations ---------------------------%
[T,nvars]= size(z);          % sample size    %
% Generate the Dependent Variable Vector %%
yyVAR = z(nlags+1:T, :);
% Form the Regressors %%
xxVAR = ones(T-nlags,1);
for i = 1:nlags
    xxVAR = [xxVAR, z((nlags+1-i):(T-i),:)];    %"[]" : concatenation operator
end
% OLS on the VAR %%
betaVAR = (xxVAR'*xxVAR)\(xxVAR'*yyVAR); const = betaVAR(1,:);
% Get Structural Residuals %%
shocksEstimated = yyVAR - xxVAR*betaVAR;         %% Reduced form residuals 
stdWoldInnovation = std(shocksEstimated);
shocksEstimated = [NaN(nlags,nvars);shocksEstimated];    %% Padding to make it of the size as the original sample
clear xxVAR yyVAR

%companion matrix of demeaned VAR (=> don't include constant term)
M=zeros(nvars*nlags,nvars*nlags);			
M(1:nvars,:)=betaVAR(2:nvars*nlags+1,:)';
M(nvars+1:nvars*nlags,1:nvars*nlags-nvars)=eye(nvars*nlags-nvars);

% % Extract impulse vectors 
% %------------------------
% 
% %Lower triangular matrix s.t. vcv of fundamental shocks is I matrix = Cholesky decomp of vmat
% Atilde = chol(vmat)';

%compute Rtildes for each l
WoldCoeffEstimated=zeros(nvars,nvars,Hmax+1);
for l=0:Hmax
    C_l=M^l;
    WoldCoeffEstimated(:,:,l+1)=C_l(1:nvars,1:nvars);%*Atilde;
end

[zzComponents] = ComputeMVWoldComponentsFromMA(WoldCoeffEstimated,shocksEstimated,J);