%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% matlab program which does a full decomposition of any given 1-D signal 
% using dwt at each level, up to the desired level J
% Last revised Jan 2009
% 
% The non-decimated Haar algorithm is exactly the same as the a trous algorithm, except that the low-pass filter h, (1/16,1/4,3/8,1/4,1/16), is
% replaced by the simpler filter (1/2, 1/2). There h is now non-symmetric.
%  
% It returns Transform = detail and approximation coefficients
% SEE ALSO modwt(HistoSeries, 'haar', ResolutionLevel, 'circular');
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function Transform=RedundantHaar(ResolutionLevel,HistoSeries,StartP)
if isempty(StartP)
StartP = 1;
end
T=size(HistoSeries,1);

wtx = zeros(T,ResolutionLevel);
Transform = zeros(T,ResolutionLevel+1);

for lev = 1 : ResolutionLevel
    % disp(['Scale', num2str(lev)])
    if lev == 1 
        for i = StartP : T
            step_ = i - 1 - 2^(lev-1);
            if step_ < 0 
                step_ = 0;
            end
            wtx(i, lev)       = (HistoSeries(1 + step_,1) + HistoSeries(i,1)) / 2;
            Transform(i, lev) = (HistoSeries(i        ,1) - wtx(i, lev));
        end
    else
        for i = StartP : T
            step_ = i - 1 - 2 ^ (lev-1);
            if step_ < 0 
                step_ = 0;
            end
            wtx(i, lev)       = (wtx(step_ + 1, lev - 1) + wtx(i, lev - 1)) / 2;
            Transform(i, lev) = (wtx(i        , lev - 1) - wtx(i, lev));
        end
    end
end
%Residual
    for i = StartP : T
       Transform(i, ResolutionLevel+1) = wtx(i, ResolutionLevel);
    end