function [gamx0H, omegb, sige] = pirfGAUSS(z,p,h)
% /******************************************************************
% This proc computes: 
% - the reduced form projections jointly
% - gam_0_H is of size r(H+1) x r; and the impulse response of the ith variable to a shock in the jth variable at horizon h corresponds to the (i,j) element of the rxr sub-matrix gam_h.
% - their covariance matrix
% ******************************************************************/

% local TT, y, x, zz, mz,c, i, b0h, bi, v, f, sige, phib, sigv, hp, omega, omegb, b, aic, aicc, sic;
  T  = size(z,1);
  TT = T-h-p+1; % (T-plag-H)

%% Generate Dependent Variable Vector %%
  y   = z(p+1:T  ,:);
% y   = z(p  :T-1,:); % BASED ON ORIGINAL JORDA CODE, I prefer to add EYE(r) at the end of estimation, instead
% // Generate the vector of endogenous variables
% y = z[p:T-h,.];
% i = 1;
%     do while i le h;
%         y = y~z[(p+i):T-h+i,.];
%         i = i+1;   
%     endo;

%% Generate the First Lag %%
x1  = z(p  :T-1,:) ;      rr = cols(x1);

%% Generate the Constant and Remaining Lags %%
x   = ones(T-p,1);      
for i = 2:p
    x = [x, z(p+1-i:T-i,:)];
end

%% Generate the Leads for the Projections %% // Generate the vector of endogenous variables
    yy_h =        y(1:size(y,1)-h+1,:);
for j = 2:h
    yy_h = [yy_h, y(j:size(y,1)-h+j,:)];
end

%% Resize the Regressors for the Lost Observations %%
xx1 = x1(1:size(x1,1)-h+1,:); % (T-plag-H) x (r* plag     ) where r=# of variables
xx  =  x(1:size(x ,1)-h+1,:); % (T-plag-H) x (r*(plag-1)+1) where r=# of variables

%% Generate the M_z matriz (Idempotent Matrix px)
% mz = eye(TT) - zz*inv(zz' zz)*zz';
  mz = eye(TT) - xx*inv(xx'*xx)*xx';

%% IRF Coefficient Matrix %%
  gamx1H = (yy_h'*mz*xx1)*inv(xx1'*mz*xx1); 
  gamx0H = [eye(rr); gamx1H];

%% Variance-Covariance of the IRF Coefficients %%
v = mz*yy_h-mz*xx1*gamx1H';   %N.B. these errors are "roughly" the same as the errors obtained from [beta, vcv, sig_a] = varols(z, p, 1); plot([errorFromVAR(1:end-h+1,ii) errorFromJorda(:,ii)]) ii=1,...,3 where errorFromJorda=v
  sige = (v(:,   1:  rr)'*v(:,   1:  rr))/TT; % see Eq.(17)
% sige = (v(:,rr+1:2*rr)'*v(:,rr+1:2*rr))/TT; % --> BASED ON ORIGINAL JORDA CODE, see pirfGAUSSFromJorda.m

phib= zeros(size(yy_h,2)+rr,rr);             % --> due to line 48
  hh  = cols(yy_h)/rr    ;                   % --> due to line 48
% hh  = cols(yy_h)/rr - 1;                   % --> BASED ON ORIGINAL JORDA CODE

for ii=1:hh                               % see Eq.(15) in Jorda(2007) "Joint Inference and Counterfactual Experimentation for Impulse Response Functions",
    phib = [phib [zeros(rr*ii,rr); gamx0H(1:rr*(hh+1-ii),:)]]; 
end

sigv  = phib*(kron(eye(hh+1),sige))*phib';% see Eq.(16) in Jorda(2007) "Joint Inference and Counterfactual Experimentation for Impulse Response Functions", it accounts for B_0 = I_r
omegb = kron(inv(xx1'*mz*xx1),sigv);      % see Proposition 2 in Jorda(2007) "Joint Inference and Counterfactual Experimentation for Impulse Response Functions",
        